from typing import Tuple
from awsee.sessioninfo import CallerIdentity
from awsee.messages import Messages
import boto3
from awsee.general import *
from dateutil import tz
from datetime import timedelta, tzinfo
from awsee.logmanager import LogManager
from awsee.style import Style
from awsee.utils import Utils
from awsee.preferences import Preferences
from botocore.exceptions import ClientError


class AwsServices:

    def __init__(self):
        self.preferences = Preferences()

    def botoSession(self, profile):
        if not profile:
           return boto3.Session()
        return boto3.Session(profile_name=profile)

    def getAccountOwner(self, profile=None):
        sts = self.botoSession(profile).client("sts")
        return sts.get_caller_identity()

    def getMFADevices(self, profile=None):
        iam = self.botoSession(profile).client("iam")
        return iam.list_mfa_devices()

    def botoSessionWithToken(self, stsToken):
        session = boto3.session.Session(
            aws_access_key_id=stsToken['Credentials']['AccessKeyId'],
            aws_secret_access_key=stsToken['Credentials']['SecretAccessKey'],
            aws_session_token=stsToken['Credentials']['SessionToken']
        )
        return session
    
    def startSessionWithTokenAndAssumeRole(self, _stsToken, roleArn) -> Tuple[dict, dict]:
        stsToken = self.botoSessionWithToken(_stsToken).client('sts').assume_role(
            RoleArn=roleArn,
            RoleSessionName="mysession",
            DurationSeconds=self.preferences.durationSeconds
        )
        callerIdentity = self.getCallerIdentity(stsToken)
        self._convertsUTCDateExpirationToLocaZoneDate(stsToken)
        return stsToken, callerIdentity


    def assumeRole(self, roleArn, profile=None, mfaSerial=None, mfaToken=None) -> Tuple[dict, dict]:
        try:
            stsToken       = None
            callerIdentity = None
            if mfaSerial and mfaToken:
                # In case a MFA Token is necessary to assume the role
                stsToken = self.botoSession(profile).client('sts').assume_role(
                    RoleArn=roleArn,
                    RoleSessionName="mysession",
                    DurationSeconds=self.preferences.durationSeconds,
                    SerialNumber=mfaSerial,
                    TokenCode=mfaToken
                )
            else:
                # In case the active AWS Session does not need a MFA Role 
                stsToken = self.botoSession(profile).client('sts').assume_role(
                    RoleArn=roleArn,
                    RoleSessionName="mysession",
                    DurationSeconds=self.preferences.durationSeconds
                )
            callerIdentity = self.getCallerIdentity(stsToken)
        except ClientError as e:
            if e.response['Error']['Code'] == "AccessDenied":
                msg = e.response['Error']['Message']
                if not mfaSerial:
                    mfaSerial = ""
                Messages.showError("Access Denied!",f"{msg}\n    {mfaSerial}")
                return None, None
            else:
                print(e.response['Error']['Code'])
                raise e
        self._convertsUTCDateExpirationToLocaZoneDate(stsToken)
        return stsToken, callerIdentity


    def _convertsUTCDateExpirationToLocaZoneDate(self, stsToken):
        localDate = Utils.parseUTCDateToLocalZoneDate(stsToken["Credentials"]["Expiration"])
        stsToken["Credentials"]["Expiration"] = localDate

    def getCallerIdentity(self, _stsToken):
        try:
            callerIdentity = self.botoSessionWithToken(_stsToken)  \
                                  .client('sts').get_caller_identity()
            return callerIdentity
        except ClientError as e:
            if e.response['Error']['Code'] == "AccessDenied":
                msg = e.response['Error']['Message']
                Messages.showError("Access Denied!",f"{msg}")
                return None
            else:
                print(e.response['Error']['Code'])
                raise e

    def getSessionToken(self, mfaSerial, mfaToken, profile=None) -> Tuple[dict, dict]:
        try:
            session = self.botoSession(profile)
            stsToken = session.client('sts').get_session_token(
                DurationSeconds=self.preferences.durationSeconds,
                SerialNumber=mfaSerial,
                TokenCode=mfaToken,
            )
            callerIdentity = self.getCallerIdentity(stsToken)
        except ClientError as e:
            if e.response['Error']['Code'] == "AccessDenied":
                msg = e.response['Error']['Message']
                Messages.showError("Access Denied!",f"{msg}\n    {mfaSerial}")
                return None, None
            else:
                print(e.response['Error']['Code'])
                raise e
        self._convertsUTCDateExpirationToLocaZoneDate(stsToken)
        return stsToken, callerIdentity
        
        # For Temporary Tests
        # stsToken = {
        #     "AccessKeyId": "ASIAV7NRRSDFSFFGGQSP24HZ",
        #     "SecretAccessKey": "QXsks0IQyTnS9qkUntDtrukfsldhyfnwlefGB8ztFZ7",
        #     "SessionToken": "FwoGZXIvYXdzEJ3//////////wEaDDVEVIvDLPKruvdC4CKFAQ7hnQi0eWG7pjprKnsmFNAnvgMtP4foPy6KygfDcBrEExJdWxnL0S4ok=",
        #     "Expiration": datetime.datetime.now()
        # }
        # return stsToken

        
        



import json, datetime, os
import clipboard
import pytz
from io import StringIO
from datetime import timezone
from dateutil import tz
from os import stat
from pygments import highlight, lexers, formatters
from awsee.style import Style
from awsee.general import *


class Utils:
    
    @staticmethod
    def time_to_expire(expDate):
        result = {}

        total_seconds = (expDate - datetime.datetime.now(timezone.utc)).seconds
        total_minutes = total_seconds / 60
        total_hours   = total_minutes / 60

        result["total_seconds"] = total_seconds
        result["total_minutes"] = total_minutes
        result["total_hours"]   = total_hours

        hours   = total_minutes / 60
        minutes = ((total_minutes / 60) - int(hours)) * 60
        seconds = ((((total_minutes / 60) - int(hours)) * 60) -  int(minutes)) * 60
        hours   = int(hours)
        minutes = int(minutes)
        seconds = int(seconds)
        result["HH:MM:SS"] = f"{hours:02.0f}h:{minutes:02.0f}m:{seconds:02.0f}s"

        return result
    
    @staticmethod
    def label_time_to_expire(expDate, format="min"):
        time_to_expire = Utils.time_to_expire(expDate, format)


    @staticmethod
    def formatPrettyJson(dictResult):
        jsonResult = Utils.dictToJson(dictResult)
        colorful_json = highlight(jsonResult, lexers.JsonLexer(), formatters.TerminalFormatter())
        return colorful_json
    
    @staticmethod 
    def dictToJson(response, sortKeys=False):
        strObj = StringIO()
        json.dump(response, strObj, default=str, indent=4, sort_keys=sortKeys)
        return strObj.getvalue()
    
    @staticmethod 
    def cleanBreakLines(text):
        if isinstance(text, str):
           text = text.replace("\n","")
           return text
        return text
    
    @staticmethod
    def addToClipboard(text):
        try:
            textNoColor = Utils.removeCharsColors(text)
            if textNoColor:
               clipboard.copy(textNoColor.strip())
        except:
            pass
    
    @staticmethod
    def isRunningOnGitBash():
        try:
            # Probably on a GitBash environment (Windows)
            os.environ["SHELL"]
            return True
        except:
            return False

    @staticmethod
    def isWindows():
        return 1 if os.name == "nt" else 0
    
    @staticmethod
    def parseUTCDateToLocalZoneDate(utc_date):
        #utc_date       = datetime.datetime.strptime(string_date, "%Y-%m-%d %H:%M:%S+00:00")
        local_timezone = tz.tzlocal() 
        local_datetime = utc_date.replace(tzinfo=pytz.utc)
        local_datetime = local_datetime.astimezone(local_timezone)
        return local_datetime
        

    @staticmethod
    def isNumber(s):
        try:
            int(s)
            return True
        except ValueError:
            return False
    
    @staticmethod
    def removeCharsColors(text):
        if isinstance(text, str):
           text = text.replace(Style.UNDERLINE,"") \
                      .replace(Style.RESET,"") \
                      .replace(Style.BLACK,"") \
                      .replace(Style.RED,"") \
                      .replace(Style.GREEN,"") \
                      .replace(Style.YELLOW,"") \
                      .replace(Style.BLUE,"") \
                      .replace(Style.MAGENTA,"") \
                      .replace(Style.CYAN,"") \
                      .replace(Style.WHITE,"") \
                      .replace(Style.BBLACK,"") \
                      .replace(Style.BRED,"") \
                      .replace(Style.BGREEN,"") \
                      .replace(Style.BYELLOW,"") \
                      .replace(Style.BBLUE,"") \
                      .replace(Style.BMAGENTA,"") \
                      .replace(Style.BCYAN,"") \
                      .replace(Style.BWHITE,"") \
                      .replace(Style.BG_BLACK,"") \
                      .replace(Style.BG_RED,"") \
                      .replace(Style.BG_GREEN,"") \
                      .replace(Style.BG_YELLOW,"") \
                      .replace(Style.BG_BLUE,"") \
                      .replace(Style.BG_PURPLE,"") \
                      .replace(Style.BG_CYAN,"") \
                      .replace(Style.BG_WHITE,"") \
                      .replace(Style.IBLACK,"") \
                      .replace(Style.IRED,"") \
                      .replace(Style.IGREEN,"") \
                      .replace(Style.IYELLOW,"") \
                      .replace(Style.IBLUE,"") \
                      .replace(Style.IMAGENTA,"") \
                      .replace(Style.ICYAN,"") \
                      .replace(Style.IWHITE,"") \
                      .replace(Style.BIBLACK,"") \
                      .replace(Style.BIRED,"") \
                      .replace(Style.BIGREEN,"") \
                      .replace(Style.BIYELLOW,"") \
                      .replace(Style.BIBLUE,"") \
                      .replace(Style.BIPURPLE,"") \
                      .replace(Style.BICYAN,"") \
                      .replace(Style.BIWHITE,"") \
                      .replace(Style.On_IBLACK,"") \
                      .replace(Style.On_IRED,"") \
                      .replace(Style.On_IGREEN,"") \
                      .replace(Style.On_IYELLOW,"") \
                      .replace(Style.On_IBLUE,"") \
                      .replace(Style.On_IPURPLE,"") \
                      .replace(Style.On_ICYAN,"") \
                      .replace(Style.On_IWHITE,"")
        return text



import os
import configparser
from os.path import expanduser
from awsee.emoticons import Emoticons
from awsee.utils import Utils
from awsee.general import *

FILE_INI                  = os.path.join(expanduser("~"),".awsee","awsee.ini")
GENERAL                   = "GENERAL"
WINDOWS                   = "WINDOWS"
LINUX                     = "LINUX"
GENERAL_DEFAULT_PROFILE   = "default-profile"
GENERAL_EMOTICONS_ENABLED = "emoticons-enabled"
GENERAL_NO_COLOR          = "no-color"
GENERAL_DURATION_SECONDS  = "duration-seconds"
WINDOWS_TERMINAL_COMMAND  = "terminal-command"
LINUX_TERMINAL_COMMAND    = "terminal-command"

class Preferences:

    

    @property
    def defaultProfile(self):
        return self._defaultProfile
    
    @property
    def emoticonsEnabled(self):
        return self._emoticonsEnabled
    
    @property
    def noColor(self):
        return self._noColor

    @property
    def windows(self):
        return self._windows

    @property
    def linux(self):
        return self._linux
    
    @property
    def durationSeconds(self):
        return self._duration_seconds

    def __init__(self):
        if not os.path.exists(FILE_INI):
            configFileIni = configparser.ConfigParser(allow_no_value=True)
            configFileIni.add_section(GENERAL)
            configFileIni.add_section(WINDOWS)
            configFileIni.add_section(LINUX)
            configFileIni.set(GENERAL, GENERAL_DEFAULT_PROFILE,"default")
            configFileIni.set(GENERAL, GENERAL_EMOTICONS_ENABLED,"true")
            configFileIni.set(GENERAL, GENERAL_NO_COLOR,"false")
            configFileIni.set(GENERAL, "; Duration, in seconds, of the role session. Range from 900 seconds (15 minutes) up to the maximum session duration that is set for the role (1h to 12h).")
            configFileIni.set(GENERAL, "; If you specify a value higher than this setting or the administrator setting (whichever is lower), the operation fails.")
            configFileIni.set(GENERAL, GENERAL_DURATION_SECONDS,"3600")

            configFileIni.set(WINDOWS, "; WINDOWS Options for New Terminal Window")
            configFileIni.set(WINDOWS, ";   Leave it blanks to use the default options CMDER or CMD")
            configFileIni.set(WINDOWS, ";   To use Git-Bash, use option 1 or 2 pointing to your installation path")
            configFileIni.set(WINDOWS, ";   To use Windows Terminal, use option 3")
            configFileIni.set(WINDOWS, "; Option 0) terminal-command =")
            configFileIni.set(WINDOWS, "; Option 1) terminal-command = C:\\Developer\\Git\\apps\\Git\\bin\\sh.exe")
            configFileIni.set(WINDOWS, "; Option 2) terminal-command = C:\\Developer\\Git\\apps\\Git\\git-bash.exe")
            configFileIni.set(WINDOWS, "; Option 3) terminal-command = wt")
            configFileIni.set(WINDOWS, WINDOWS_TERMINAL_COMMAND,"")

            configFileIni.set(LINUX, "; LINUX Options for New Terminal Window")
            configFileIni.set(LINUX, ";   Leave it blanks to use the default option (Still to be worked, not ready!)")
            configFileIni.set(LINUX, ";   ---")
            configFileIni.set(LINUX, "; Option 0) terminal-command =")
            configFileIni.set(LINUX, "; Option 1) terminal-command = ")
            configFileIni.set(LINUX, LINUX_TERMINAL_COMMAND ,"")

            with open(FILE_INI,'w') as configfile:
                configFileIni.write(configfile)
                configFileIni = configparser.ConfigParser(allow_no_value=True)
        else:
            # Add new configuration parameters (for those who have init file already created on the his/her system after)
            self.check_add_new_parameter(GENERAL, GENERAL_DURATION_SECONDS, 3600)
        
        configFileIni = configparser.ConfigParser()
        configFileIni.read(FILE_INI)
        self._defaultProfile   = configFileIni[GENERAL][GENERAL_DEFAULT_PROFILE]
        self._emoticonsEnabled = configFileIni[GENERAL][GENERAL_EMOTICONS_ENABLED] in ['True','true']
        
        if GENERAL_NO_COLOR in configFileIni[GENERAL]:
           self._noColor       = configFileIni[GENERAL][GENERAL_NO_COLOR] in ['True','true'] 
        else:
           self._noColor       = False

        if GENERAL_DURATION_SECONDS in configFileIni[GENERAL]:
           _vlr = configFileIni[GENERAL][GENERAL_DURATION_SECONDS]
           if Utils.isNumber(_vlr):
              self._duration_seconds = int(_vlr)
           else:
              self._duration_seconds = 3600 
        else:
           self._duration_seconds = 3600

        self._windows          = Windows(configFileIni[WINDOWS][WINDOWS_TERMINAL_COMMAND])
        self._linux            = Linux(configFileIni[LINUX][LINUX_TERMINAL_COMMAND])

        Emoticons.ENABLED = self._emoticonsEnabled
    
    def check_add_new_parameter(self, section, new_parameter, default_parameter_value):
        """ Add new configuration parameter (for those who have init their files already, after this parameter were added) """
        with open(FILE_INI, 'r') as file:
            data_ini = file.read()
        
        with open(FILE_INI, 'r+') as f:
            write_back = False
            lines = f.readlines()
            for i, line in enumerate(lines):
                if line.startswith(f'[{section}]'):
                    if not new_parameter in data_ini:
                        lines[i] = lines[i].strip() + f'\n{new_parameter} = {default_parameter_value}\n'
                        write_back = True
            if write_back:
                f.seek(0)
                for line in lines:
                    f.write(line)

class Windows:

    @property
    def terminalCommand(self):
        return self._terminal_command

    def __init__(self, _terminal_command):
        self._terminal_command = _terminal_command

class Linux:

    @property
    def terminalCommand(self):
        return self._terminal_command

    def __init__(self, _terminal_command):
        self._terminal_command = _terminal_command
